@@ -240,8 +240,8 @@ append_key_value (gpointer key, gpointer value, gpointer bson_string)
 size_t
 server_response (char* ptr, size_t size, size_t nmemb, void* s)
 {
-    struct response_string* resp = (struct response_string*) s;
-    grow_response_string (resp, ptr, size * nmemb);
+    GString *resp = (GString *) s;
+    g_string_append (resp, ptr);
     return size * nmemb;
 }
 
@@ -295,7 +295,7 @@ bsonify (GHashTable* report, bson* b, const char** bson_message,
 }
 
 int
-upload_report (const char* message_data, int message_len, struct response_string* s)
+upload_report (const char* message_data, int message_len, GString *s)
 {
     CURL* curl = NULL;
     CURLcode result_code = 0;
@@ -374,8 +374,8 @@ parse_report (const char* report_path, gboolean full_report, GError** error)
     gchar* value_p = NULL;
     GError* err = NULL;
     gchar* end = NULL;
-    int value_length;
-    int value_pos;
+    size_t value_length;
+    size_t value_pos;
 
     g_return_val_if_fail (report_path, NULL);
 
@@ -562,7 +562,7 @@ upload_core (const char* uuid, const char* arch, const char* core_data) {
     long response_code = 0;
     struct curl_slist* list = NULL;
     char* crash_db_core_url = NULL;
-    struct response_string s;
+    GString *s = NULL;
 
 
     g_return_val_if_fail (uuid, FALSE);
@@ -580,7 +580,7 @@ upload_core (const char* uuid, const char* arch, const char* core_data) {
         g_free (crash_db_core_url);
         return FALSE;
     }
-    init_response_string (&s);
+    s = g_string_new (NULL);
     curl_easy_setopt (curl, CURLOPT_POST, 1);
     curl_easy_setopt (curl, CURLOPT_NOPROGRESS, 1);
     list = curl_slist_append (list, "Content-Type: application/octet-stream");
@@ -589,7 +589,7 @@ upload_core (const char* uuid, const char* arch, const char* core_data) {
     curl_easy_setopt (curl, CURLOPT_HTTPHEADER, list);
     curl_easy_setopt (curl, CURLOPT_POSTFIELDS, (void*)core_data);
     curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, server_response);
-    curl_easy_setopt (curl, CURLOPT_WRITEDATA, &s);
+    curl_easy_setopt (curl, CURLOPT_WRITEDATA, s);
     curl_easy_setopt (curl, CURLOPT_VERBOSE, 0L);
     curl_easy_setopt (curl, CURLOPT_SSL_VERIFYPEER, verifypeer);
 
@@ -603,7 +603,7 @@ upload_core (const char* uuid, const char* arch, const char* core_data) {
     log_msg ("Response code: %ld\n", response_code);
     curl_easy_cleanup (curl);
     g_free (crash_db_core_url);
-    destroy_response_string (&s);
+    g_string_free (s, TRUE);
 
     return result_code == CURLE_OK && response_code == 200;
 }
@@ -667,7 +667,7 @@ parse_and_upload_report (const char* crash_file)
     gboolean success = FALSE;
     int message_len = 0;
     const char* message_data = NULL;
-    struct response_string s;
+    GString* s = NULL;
     GError* error = NULL;
     bson b[1];
     int response = 0;
@@ -694,8 +694,8 @@ parse_and_upload_report (const char* crash_file)
         success = TRUE;
     } else {
         log_msg ("Uploading %s.\n", crash_file);
-        init_response_string (&s);
-        response = upload_report (message_data, message_len, &s);
+        s = g_string_new (NULL);
+        response = upload_report (message_data, message_len, s);
         if (bson_data (b))
             bson_destroy (b);
 
@@ -710,14 +710,14 @@ parse_and_upload_report (const char* crash_file)
 
         if (response > 200) {
             log_msg ("Server replied with:\n");
-            log_msg ("%s\n", s.p);
+            log_msg ("%s\n", s->str);
         }
 
-        if (response == 200 && s.length > 0)
-            handle_response (report, s.p);
-        destroy_response_string (&s);
+        if (response == 200 && s->len > 0)
+            handle_response (report, s->str);
     }
 
+    g_string_free (s, TRUE);
     g_hash_table_foreach (report, destroy_key_and_value, NULL);
     g_hash_table_destroy (report);
 
